///////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright 2018 Leszek Koltunski  leszek@koltunski.pl                                          //
//                                                                                               //
// This file is part of Distorted.                                                               //
//                                                                                               //
// This library is free software; you can redistribute it and/or                                 //
// modify it under the terms of the GNU Lesser General Public                                    //
// License as published by the Free Software Foundation; either                                  //
// version 2.1 of the License, or (at your option) any later version.                            //
//                                                                                               //
// This library is distributed in the hope that it will be useful,                               //
// but WITHOUT ANY WARRANTY; without even the implied warranty of                                //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                             //
// Lesser General Public License for more details.                                               //
//                                                                                               //
// You should have received a copy of the GNU Lesser General Public                              //
// License along with this library; if not, write to the Free Software                           //
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA                //
///////////////////////////////////////////////////////////////////////////////////////////////////

precision highp float;
precision highp int;

in vec2 v_TexCoordinate;
in vec2 v_Pixel;              // location of the current fragment, in pixels

//////////////////////////////////////////////////////////////////////////////////////////////
// per-pixel linked list. Order Independent Transparency.

uniform uvec2 u_Size;

layout (std430,binding=1) buffer linkedlist  // first (u_Size.x*u_Size.y) uints - head pointers,
  {                                          // one for each pixel in the Output rectangle.
  uint u_Records[];                          //
  };                                         // Next 3*u_numRecords uints - actual linked list, i.e.
                                             // triplets of (pointer,depth,rgba).

//////////////////////////////////////////////////////////////////////////////////////////////
// Pass1 of the OIT algorithm - 'clear the head pointers' phase.
// No we cannot optimize this out by moving the 'u_Records[index]=0u' to the end of the Pass3,
// because between passes the size of the surface we render to might change.

void main()                    		
  {
  uint index= uint(v_Pixel.x) + uint(v_Pixel.y) * u_Size.x;
  u_Records[index] = 0u;
  discard;
  }